package com.bpcl.lpg.core.service.impl;

import org.slf4j.LoggerFactory;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Iterator;
import java.io.Writer;
import java.io.BufferedWriter;
import java.io.FileWriter;
import org.apache.commons.collections4.CollectionUtils;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import de.hybris.platform.core.model.media.MediaModel;
import java.text.MessageFormat;
import org.apache.commons.lang3.StringUtils;
import java.util.Objects;
import java.util.Collection;
import com.bpcl.lpg.core.model.CashMemoModel;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Date;
import com.bpcl.lpg.core.model.CashMemoFileTrackerModel;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import de.hybris.platform.servicelayer.exceptions.AmbiguousIdentifierException;
import de.hybris.platform.servicelayer.exceptions.UnknownIdentifierException;
import java.io.File;
import javax.annotation.Resource;
import com.bpcl.core.ym.notication.service.BpclRetailYMV2NotificationEngineService;
import java.util.List;
import com.bpcl.lpg.core.utill.LpgCashMemoUtil;
import com.bpcl.lpg.core.validator.LpgCashMemoImportValidator;
import com.bpcl.lpg.core.populator.LpgCashMemoImportPopulator;
import com.bpcl.core.ym.notication.service.BpclReportEmailService;
import com.bpcl.lpg.core.dao.LpgDeliveryConfirmationDao;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.service.LpgCashMemoImportService;

public class LpgCashMemoImportServiceImpl implements LpgCashMemoImportService
{
    private static final Logger LOG;
    private static ModelService modelService;
    private static ConfigurationService configurationService;
    private static LpgDeliveryConfirmationDao lpgDeliveryConfirmationDao;
    private static BpclReportEmailService bpclReportEmailService;
    private static LpgCashMemoImportPopulator lpgCashMemoImportPopulator;
    private static LpgCashMemoImportValidator lpgCashMemoImportValidator;
    private static LpgCashMemoUtil lpgCashMemoUtil;
    private static String line;
    private static String splitBy;
    private static String[] header;
    private static String[] errorFileHeader;
    private static List<String[]> errorsList;
    private static String errorDetails;
    private static boolean headerData;
    private static int i;
    private static String CSV_FILE_NAME;
    private static String ERROR_FILE_CSV_NAME;
    private static String ERROR_FILE_CSV_WITH_PATH;
    private static final String EMAIL_LINE_1 = "Please note errors are observed while reading CashMemo file ";
    private static final String EMAIL_LINE_2 = "A separate file with error records observed is enclosed as per attachment. Total number of error records : ";
    private static final String EMAIL_LINE_3 = "This is for information and necessary action at your end pls.";
    private static final String SEPARATOR = ",";
    @Resource(name = "bpclRetailYMV2NotificationEngineService")
    private BpclRetailYMV2NotificationEngineService bpclRetailYMV2NotificationEngineService;
    
    public void processInboundFile(final File file) {
        try {
            this.readAndProcessCSV(file);
        }
        catch (final UnknownIdentifierException ex) {
            LpgCashMemoImportServiceImpl.LOG.info("UnknownIdentifierException : {}", (Throwable)ex);
        }
        catch (final AmbiguousIdentifierException ex2) {
            LpgCashMemoImportServiceImpl.LOG.info("AmbiguousIdentifierException : {}", (Throwable)ex2);
        }
        catch (final ParseException ex3) {
            LpgCashMemoImportServiceImpl.LOG.info("ParseException : {}", ex3);
        }
        catch (final IOException ex4) {
            LpgCashMemoImportServiceImpl.LOG.info("IOException : {}", ex4);
        }
        catch (final NumberFormatException ex5) {
            LpgCashMemoImportServiceImpl.LOG.info("NumberFormatException : {}", ex5);
        }
        catch (final Exception ex6) {
            LpgCashMemoImportServiceImpl.LOG.info("Exception : {}", ex6);
        }
    }
    
    public void readAndProcessCSV(final File file) throws ParseException, IOException {
        LpgCashMemoImportServiceImpl.LOG.info("Step 1: readAndProcess cashmemo csvFile");
        final BufferedReader br = new BufferedReader(new FileReader(file));
        LpgCashMemoImportServiceImpl.CSV_FILE_NAME = file.getName();
        final CashMemoFileTrackerModel cashMemoFileTrackerModel = (CashMemoFileTrackerModel)LpgCashMemoImportServiceImpl.modelService.create((Class)CashMemoFileTrackerModel.class);
        cashMemoFileTrackerModel.setCashMemoFileName(LpgCashMemoImportServiceImpl.CSV_FILE_NAME);
        cashMemoFileTrackerModel.setProcessStartTimeStamp(new Date());
        LpgCashMemoImportServiceImpl.modelService.save((Object)cashMemoFileTrackerModel);
        LpgCashMemoImportServiceImpl.LOG.info("Step 2: Initial Timestamp : " + cashMemoFileTrackerModel.getProcessStartTimeStamp());
        LpgCashMemoImportServiceImpl.i = 2;
        LpgCashMemoImportServiceImpl.errorsList = new ArrayList<String[]>();
        LpgCashMemoImportServiceImpl.headerData = true;
        while ((LpgCashMemoImportServiceImpl.line = br.readLine()) != null) {
            if (LpgCashMemoImportServiceImpl.headerData) {
                LpgCashMemoImportServiceImpl.LOG.info("Step 3: Read Header Row of cashmemo :\n" + LpgCashMemoImportServiceImpl.line.toString());
                LpgCashMemoImportServiceImpl.header = LpgCashMemoImportServiceImpl.line.split(LpgCashMemoImportServiceImpl.splitBy);
                (LpgCashMemoImportServiceImpl.errorFileHeader = Arrays.copyOf(LpgCashMemoImportServiceImpl.header, LpgCashMemoImportServiceImpl.header.length + 1))[LpgCashMemoImportServiceImpl.errorFileHeader.length - 1] = "ErrorDetails";
                LpgCashMemoImportServiceImpl.headerData = false;
                LpgCashMemoImportServiceImpl.LOG.info("Step 4: Started Data Validation for cashmemo rows :");
            }
            else {
                LpgCashMemoImportServiceImpl.errorDetails = "";
                final String[] cashmemo = LpgCashMemoImportServiceImpl.line.split(LpgCashMemoImportServiceImpl.splitBy);
                if (cashmemo.length <= 0) {
                    break;
                }
                String[] cashData;
                if (cashmemo.length < LpgCashMemoImportServiceImpl.header.length && cashmemo.length > 0) {
                    final int empty = LpgCashMemoImportServiceImpl.header.length - cashmemo.length;
                    cashData = Arrays.copyOf(cashmemo, LpgCashMemoImportServiceImpl.header.length);
                    for (int p = cashmemo.length; p < LpgCashMemoImportServiceImpl.header.length; ++p) {
                        cashData[p] = "";
                    }
                }
                else {
                    cashData = cashmemo;
                }
                if (this.validateBasicData(cashData)) {
                    this.validateBasedOnAction(cashData, cashMemoFileTrackerModel.getProcessStartTimeStamp());
                }
                ++LpgCashMemoImportServiceImpl.i;
            }
        }
        if (LpgCashMemoImportServiceImpl.errorsList.size() > 0) {
            LpgCashMemoImportServiceImpl.LOG.info("Final Step : No.of Error Rows occured in the sheet: " + LpgCashMemoImportServiceImpl.errorsList.size());
            this.createCsvFileAndEmail(LpgCashMemoImportServiceImpl.errorFileHeader, LpgCashMemoImportServiceImpl.errorsList);
        }
        cashMemoFileTrackerModel.setProcessEndTimeStamp(new Date());
        LpgCashMemoImportServiceImpl.modelService.save((Object)cashMemoFileTrackerModel);
        LpgCashMemoImportServiceImpl.LOG.info("Final Timestamp : " + cashMemoFileTrackerModel.getProcessEndTimeStamp());
    }
    
    public void validateBasedOnAction(final String[] cashmemo, final Date creationDate) throws ParseException {
        final char action = cashmemo[36].charAt(0);
        LpgCashMemoImportServiceImpl.LOG.info("Inside validateBasedOnAction Action is :" + action);
        LpgCashMemoImportServiceImpl.errorDetails = "";
        switch (action) {
            case 'A': {
                LpgCashMemoImportServiceImpl.LOG.info("Inside Action A for the rowNo: " + LpgCashMemoImportServiceImpl.i);
                final CashMemoModel existingCashmemo = LpgCashMemoImportServiceImpl.lpgDeliveryConfirmationDao.getCashMemoByRefillId(Integer.parseInt(cashmemo[0]), cashmemo[1], false);
                final CashMemoModel cashMemoModel = this.validateObjectAndGenerateCashMemoModel(cashmemo, LpgCashMemoImportServiceImpl.header);
                if (null != cashMemoModel) {
                    if (this.insertCashMemo(existingCashmemo, cashMemoModel)) {
                        cashMemoModel.setCashMemoDate(creationDate);
                        LpgCashMemoImportServiceImpl.modelService.save((Object)cashMemoModel);
                    }
                    LpgCashMemoImportServiceImpl.LOG.info("CashMemoModel for Action A no.of records to save: {} ", cashMemoModel);
                }
                break;
            }
            case 'D': {
                LpgCashMemoImportServiceImpl.LOG.info("Inside Action D for the rowNo: " + LpgCashMemoImportServiceImpl.i);
                final CashMemoModel existingCashmemo = LpgCashMemoImportServiceImpl.lpgDeliveryConfirmationDao.getCashMemoByRefillId(Integer.parseInt(cashmemo[0]), cashmemo[1], false);
                if (null != existingCashmemo && existingCashmemo.getDeliveryStatus() == 0) {
                    existingCashmemo.setAction(Character.valueOf('D'));
                    LpgCashMemoImportServiceImpl.modelService.save((Object)existingCashmemo);
                    LpgCashMemoImportServiceImpl.LOG.info("CashMemoModel for Action D no.of records to save: " + existingCashmemo);
                }
                break;
            }
            case 'E': {
                LpgCashMemoImportServiceImpl.LOG.info("Inside Action E for the rowNo: " + LpgCashMemoImportServiceImpl.i);
                List<CashMemoModel> existingCashMemoList = new ArrayList<CashMemoModel>();
                final List<CashMemoModel> cashMemoRowList = new ArrayList<CashMemoModel>();
                existingCashMemoList = LpgCashMemoImportServiceImpl.lpgDeliveryConfirmationDao.getCashMemoByDistId(Integer.parseInt(cashmemo[0]), false);
                existingCashMemoList.stream().forEach(cashMemoData -> {
                    cashMemoData.setDayEnd(Boolean.valueOf(true));
                    cashMemoRowList.add(cashMemoData);
                    return;
                });
                LpgCashMemoImportServiceImpl.modelService.saveAll((Collection)cashMemoRowList);
                LpgCashMemoImportServiceImpl.LOG.info("CashMemoModel for Action E no.of records to save: " + cashMemoRowList.size());
                break;
            }
        }
    }
    
    private boolean insertCashMemo(final CashMemoModel existingCashMemo, final CashMemoModel newCashMemo) {
        return Objects.isNull(existingCashMemo);
    }
    
    public boolean validateBasicData(final String[] cashMemo) {
        final String[] actionCode = { "A", "D", "E" };
        final StringBuilder errorMsgBuilder = new StringBuilder();
        if (!Arrays.asList(actionCode).contains(cashMemo[36])) {
            this.setErrorMsgToStringBuilder(errorMsgBuilder, "Action");
        }
        if (!LpgCashMemoImportServiceImpl.lpgCashMemoUtil.isNumeric(cashMemo[0], 6, "distCode")) {
            this.setErrorMsgToStringBuilder(errorMsgBuilder, "DistCode");
        }
        if (StringUtils.equalsIgnoreCase(cashMemo[36], actionCode[0]) && StringUtils.equalsIgnoreCase(cashMemo[36], actionCode[1]) && !LpgCashMemoImportServiceImpl.lpgCashMemoUtil.isNumeric(cashMemo[1], 0, "RefillId")) {
            this.setErrorMsgToStringBuilder(errorMsgBuilder, "RefillId");
        }
        if (Objects.nonNull(errorMsgBuilder) && StringUtils.isNotEmpty(errorMsgBuilder.toString())) {
            LpgCashMemoImportServiceImpl.LOG.info("Error Step: Error found in cashmemo row :" + LpgCashMemoImportServiceImpl.i + " for the " + errorMsgBuilder.toString());
            String[] error = cashMemo;
            error = Arrays.copyOf(cashMemo, cashMemo.length + 1);
            error[error.length - 1] = "Error occurred at the Row number " + (LpgCashMemoImportServiceImpl.i + 1) + " for the " + errorMsgBuilder.toString();
            LpgCashMemoImportServiceImpl.errorsList.add(error);
            return false;
        }
        return true;
    }
    
    private void setErrorMsgToStringBuilder(final StringBuilder errorDetailsBuilder, final String errorMsg) {
        if (Objects.nonNull(errorDetailsBuilder) && StringUtils.isNotEmpty(errorDetailsBuilder.toString())) {
            errorDetailsBuilder.append(",").append(errorMsg);
        }
        else {
            errorDetailsBuilder.append(errorMsg);
        }
    }
    
    public void createCsvFileAndEmail(final String[] errorFileHeader, final List<String[]> errorsDataList) throws IOException {
        final String users = LpgCashMemoImportServiceImpl.configurationService.getConfiguration().getString("lpg.sftp.delivery.confirmation.inbound.error.email.recipient");
        final String emailSubject = MessageFormat.format(LpgCashMemoImportServiceImpl.configurationService.getConfiguration().getString("bpcl.cashmemo.inbound.error.email.subject"), LpgCashMemoImportServiceImpl.CSV_FILE_NAME);
        final String bodyContent = this.getBodyForErrorEmail(LpgCashMemoImportServiceImpl.CSV_FILE_NAME, errorsDataList.size());
        this.generateErrorCSVFile(errorFileHeader, errorsDataList);
        LpgCashMemoImportServiceImpl.LOG.info("Name of the Error File : " + LpgCashMemoImportServiceImpl.ERROR_FILE_CSV_NAME);
        final File csvErrorFile = new File(LpgCashMemoImportServiceImpl.ERROR_FILE_CSV_NAME);
        final MediaModel attachment = LpgCashMemoImportServiceImpl.lpgCashMemoUtil.converFileToMedia(csvErrorFile, ".csv");
        final List<MediaModel> mediaList = new ArrayList<MediaModel>();
        mediaList.add(attachment);
        final List<String> recipients = new ArrayList<String>();
        final List<String> recipientList = Stream.of(users.split(",")).collect((Collector<? super String, ?, List<String>>)Collectors.toList());
        recipientList.stream().forEach(user -> recipients.add(user));
        LpgCashMemoImportServiceImpl.LOG.info("sending email with file");
        if (CollectionUtils.isNotEmpty((Collection)recipients)) {
            this.bpclRetailYMV2NotificationEngineService.sendEmailV2(users, bodyContent, (List)mediaList, emailSubject, "BPCL");
        }
    }
    
    private String getBodyForErrorEmail(final String fileName, final int noOfErrors) {
        final StringBuilder emailBody = new StringBuilder("<html><body>").append("<p>");
        final String emailString = "Please note errors are observed while reading CashMemo file " + fileName + "<br/>A separate file with error records observed is enclosed as per attachment. Total number of error records : " + noOfErrors + "<br/><br/>This is for information and necessary action at your end pls.";
        emailBody.append(emailString).append("</p>").append("</body></html>");
        return emailBody.toString();
    }
    
    public void generateErrorCSVFile(final String[] headerInformation, final List<String[]> errorsDataList) {
        LpgCashMemoImportServiceImpl.ERROR_FILE_CSV_NAME = this.getReportFileName(".csv", LpgCashMemoImportServiceImpl.configurationService.getConfiguration().getString("bpcl.cashmemo.report.error.filename"));
        LpgCashMemoImportServiceImpl.ERROR_FILE_CSV_WITH_PATH = LpgCashMemoImportServiceImpl.configurationService.getConfiguration().getString("bpcl.cashmemo.hybris.input.error.filepath") + LpgCashMemoImportServiceImpl.ERROR_FILE_CSV_NAME;
        LpgCashMemoImportServiceImpl.LOG.info("Started creating Error file in CSV format ");
        try (final BufferedWriter writer = new BufferedWriter(new FileWriter(LpgCashMemoImportServiceImpl.ERROR_FILE_CSV_NAME))) {
            this.writeLine(writer, headerInformation);
            for (final String[] errorData : errorsDataList) {
                this.writeLine(writer, errorData);
            }
            LpgCashMemoImportServiceImpl.LOG.info("Error validation CSV file has been created successfully!");
        }
        catch (final IOException e) {
            LpgCashMemoImportServiceImpl.LOG.info("An error occurred while writing to the CSV file.");
            e.printStackTrace();
        }
    }
    
    public void writeLine(final BufferedWriter writer, final String[] data) throws IOException {
        for (int i = 0; i < data.length; ++i) {
            writer.write(data[i]);
            if (i < data.length - 1) {
                writer.write(",");
            }
        }
        writer.write("\n");
    }
    
    public String getReportFileName(final String ext, final String filename) {
        final Date date = Calendar.getInstance().getTime();
        final DateFormat dateFormat = new SimpleDateFormat("yyyyMMdd_hhmmss");
        final String strDate = dateFormat.format(date);
        return filename + strDate + ext;
    }
    
    public CashMemoModel validateObjectAndGenerateCashMemoModel(final String[] cashmemo, final String[] header) throws ParseException {
        final String errorDetails = LpgCashMemoImportServiceImpl.lpgCashMemoImportValidator.validateObject(cashmemo, header, LpgCashMemoImportServiceImpl.i);
        if (StringUtils.isEmpty(errorDetails)) {
            final CashMemoModel cashMemoModel = (CashMemoModel)LpgCashMemoImportServiceImpl.modelService.create((Class)CashMemoModel.class);
            LpgCashMemoImportServiceImpl.lpgCashMemoImportPopulator.populate(cashmemo, cashMemoModel);
            LpgCashMemoImportServiceImpl.LOG.info("CashMemoModel for row " + LpgCashMemoImportServiceImpl.i + cashMemoModel.toString());
            return cashMemoModel;
        }
        LpgCashMemoImportServiceImpl.LOG.info("Validation Step: Error found in cashmemo row :" + LpgCashMemoImportServiceImpl.i + " for the " + errorDetails);
        String[] error = cashmemo;
        error = Arrays.copyOf(cashmemo, cashmemo.length + 1);
        error[error.length - 1] = "Error occurred at the Row number " + (LpgCashMemoImportServiceImpl.i + 1) + " for the " + errorDetails;
        LpgCashMemoImportServiceImpl.errorsList.add(error);
        return null;
    }
    
    public ModelService getModelService() {
        return LpgCashMemoImportServiceImpl.modelService;
    }
    
    public void setModelService(final ModelService modelService) {
        LpgCashMemoImportServiceImpl.modelService = modelService;
    }
    
    public ConfigurationService getConfigurationService() {
        return LpgCashMemoImportServiceImpl.configurationService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        LpgCashMemoImportServiceImpl.configurationService = configurationService;
    }
    
    public LpgDeliveryConfirmationDao getLpgDeliveryConfirmationDao() {
        return LpgCashMemoImportServiceImpl.lpgDeliveryConfirmationDao;
    }
    
    public void setLpgDeliveryConfirmationDao(final LpgDeliveryConfirmationDao lpgDeliveryConfirmationDao) {
        LpgCashMemoImportServiceImpl.lpgDeliveryConfirmationDao = lpgDeliveryConfirmationDao;
    }
    
    public BpclReportEmailService getBpclReportEmailService() {
        return LpgCashMemoImportServiceImpl.bpclReportEmailService;
    }
    
    public void setBpclReportEmailService(final BpclReportEmailService bpclReportEmailService) {
        LpgCashMemoImportServiceImpl.bpclReportEmailService = bpclReportEmailService;
    }
    
    public LpgCashMemoUtil getLpgCashMemoUtil() {
        return LpgCashMemoImportServiceImpl.lpgCashMemoUtil;
    }
    
    public void setLpgCashMemoUtil(final LpgCashMemoUtil lpgCashMemoUtil) {
        LpgCashMemoImportServiceImpl.lpgCashMemoUtil = lpgCashMemoUtil;
    }
    
    public void setLpgCashMemoImportPopulator(final LpgCashMemoImportPopulator lpgCashMemoImportPopulator) {
        LpgCashMemoImportServiceImpl.lpgCashMemoImportPopulator = lpgCashMemoImportPopulator;
    }
    
    public LpgCashMemoImportPopulator getLpgCashMemoImportPopulator() {
        return LpgCashMemoImportServiceImpl.lpgCashMemoImportPopulator;
    }
    
    public void setLpgCashMemoImportValidator(final LpgCashMemoImportValidator lpgCashMemoImportValidator) {
        LpgCashMemoImportServiceImpl.lpgCashMemoImportValidator = lpgCashMemoImportValidator;
    }
    
    public LpgCashMemoImportValidator getLpgCashMemoImportValidator() {
        return LpgCashMemoImportServiceImpl.lpgCashMemoImportValidator;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgCashMemoImportServiceImpl.class);
        LpgCashMemoImportServiceImpl.line = "";
        LpgCashMemoImportServiceImpl.splitBy = "~";
        LpgCashMemoImportServiceImpl.header = new String[0];
        LpgCashMemoImportServiceImpl.errorFileHeader = new String[0];
        LpgCashMemoImportServiceImpl.errorsList = new ArrayList<String[]>();
        LpgCashMemoImportServiceImpl.errorDetails = "ErrorDetails";
        LpgCashMemoImportServiceImpl.headerData = true;
        LpgCashMemoImportServiceImpl.i = 2;
        LpgCashMemoImportServiceImpl.CSV_FILE_NAME = "";
        LpgCashMemoImportServiceImpl.ERROR_FILE_CSV_NAME = "";
        LpgCashMemoImportServiceImpl.ERROR_FILE_CSV_WITH_PATH = "";
    }
}
